# $Id: Compare_sstruc.py 577 2006-10-11 16:51:51Z indraneel $

import os
import string

class Sstruc:
    def __init__(self, conf, logger):
        self.conf = conf
        self.logger = logger
        structureID = conf.get('Input', 'structureID')
        sstrucdir = conf.get('Compare', 'sstrucdir')
        sstructail = conf.get('Compare', 'sstructail')
        self.fname = sstrucdir + structureID + sstructail

    def _get_lines(self):
        if not os.path.isfile(self.fname): return []
        fh = file(self.fname)
        lines = fh.readlines()
        fh.close()
        return lines

    def _get_sec_str_lines(self):
        lines = self._get_lines()
        if not lines: return []
        for start in range(len(lines)):
            if lines[start][:3] == 'num':
                break
        start += 1
        for end in range(start, len(lines)):
            if len(lines[end]) < 4:
                break
        lines = lines[start:end]
        return lines

    def _extract_secstr(self):
        lines = self._get_sec_str_lines()
        secstrdef = {}
        for line in lines:
            chain = line[5]
            if not secstrdef.has_key(chain):
                secstrdef[chain] = {}
            id = line[6:10]
            id = int(id)
            icode = line[10]
            if not secstrdef[chain].has_key(id):
                secstrdef[chain][id] = {}
            secstr = line[20]
            if not secstrdef[chain][id].has_key(icode):
                secstrdef[chain][id][icode] = secstr
        self.secstrdef = secstrdef

    def _report(self, peptides):
        total_residues = 0
        total_pssd_H = 0
        total_pssd_E = 0
        total_other_H = 0
        total_other_E = 0
        both_H = 0
        both_E = 0
        only_pssd_H = 0
        only_pssd_E = 0
        only_other_H = 0
        only_other_E = 0
        for peptide in peptides:
            for res in peptide:
                total_residues += 1
                if res.H: total_pssd_H += 1
                if res.E: total_pssd_E += 1
                if res.compare_sstruc == 'H': total_other_H += 1
                if res.compare_sstruc == 'E': total_other_E += 1
                if res.H and res.compare_sstruc == 'H': both_H += 1
                if res.E and res.compare_sstruc == 'E': both_E += 1
                if res.H and res.compare_sstruc != 'H': only_pssd_H += 1
                if res.E and res.compare_sstruc != 'E': only_pssd_E += 1
                if res.compare_sstruc == 'H' and not res.H: only_other_H += 1
                if res.compare_sstruc == 'E' and not res.E: only_other_E += 1
        line='sstruc comparison report for '+str(total_residues)+' residues:\n'
        lines = [line]
        lines += ['sstruc total_pssd_H '+str(total_pssd_H)+'\n']
        lines += ['sstruc total_pssd_E '+str(total_pssd_E)+'\n']
        lines += ['sstruc total_other_H '+str(total_other_H)+'\n']
        lines += ['sstruc total_other_E '+str(total_other_E)+'\n']
        lines += ['sstruc both_H '+str(both_H)+'\n']
        lines += ['sstruc both_E '+str(both_E)+'\n']
        lines += ['sstruc only_pssd_H '+str(only_pssd_H)+'\n']
        lines += ['sstruc only_pssd_E '+str(only_pssd_E)+'\n']
        lines += ['sstruc only_other_H '+str(only_other_H)+'\n']
        lines += ['sstruc only_other_E '+str(only_other_E)+'\n']
        self.logger.log('compare', lines)

    ## Public Methods
    
    def compare(self, structure, peptides):
        self._extract_secstr()
        for peptide in peptides:
            chain = peptide[0].get_parent().id
            if self.secstrdef.has_key(chain):
                for res in peptide:
                    if self.secstrdef[chain].has_key(res.id[1]):
                        if self.secstrdef[chain][res.id[1]].has_key(res.id[2]):
                            v = self.secstrdef[chain][res.id[1]][res.id[2]]
                            res.compare_sstruc = v
        self._report(peptides)
