# $Id: DSSPParser.py 577 2006-10-11 16:51:51Z indraneel $

# basic DSSP output parser
# Does _not_ count left handed Helix as helix
# stores in Chain Residue format and also as elements

import os, sys
import string

from Log import LogFH
from PSSDParser import Element
from DSSPElements import DSSPElements

class DSSP(dict):
    def __init__(self, logger=None):
        if logger:
            self.logger = logger
            
    def __sort(self, a, b):
        return cmp(self[self.chainID][a]['#'], self[self.chainID][b]['#'])

    def store(self, fname):
        if not os.path.isfile(fname): return 'Err'
        self.fname = os.path.basename(fname)
        fh = file(fname)
        lines = fh.readlines()
        fh.close()
        self.dsspLines = lines[:]
        for (i,line) in enumerate(lines):
            if line[2] == '#':
                break
        self.dsspHeader = lines[:i+1]
        lines = lines[i+1:]
        ctr = 0
        self.dsspSeparators = {}
        self.chainList = []
        for line in lines:
            aa = line[13]
            if aa == '!':
                #print line[:-1]
                ctr += 1
                self.dsspSeparators[chain][ctr] = line
                continue
            chain=line[11]
            resid=int(line[6:10].lstrip())
            icode=line[10]
            ss=line[16]
            handedness = line[22]
            if (ss == 'H') and (handedness == '-'):
                ss = ' '
            if not self.has_key(chain):
                self[chain]={}
                self.chainList.append(chain)
                self.dsspSeparators[chain]={}
            ctr += 1
            self[chain][(resid, icode)] = {'#':ctr, 'AA':aa, 'SS':[ss], \
                                           'line': line}
        dssp_elements = DSSPElements(self.logger, fname)
        typedef={'H':'H', 'G':'H', 'I':'H', 'E':'E'}
        dssp_elements.setType(typedef)
        self.elements = dssp_elements.getElements()

    def getSortedChains(self):
        return self.chainList

    def getSortedRes(self, chainID=None):
        if chainID: self.chainID = chainID
	residues = self[self.chainID].keys()
	residues.sort(self.__sort)
        return residues

    def show(self, outfh):
        self.chainID = self.keys()[0]
        residues = self.getSortedRes()
	for residue in residues:
	    print >> outfh, self[self.chainID][residue]['SS'][0]

if __name__ == '__main__':
    logger = LogFH('x')
    dssp = DSSP(logger)
    dssp.store(sys.argv[1])
    dssp.show(sys.stdout)
