# $Id: PSSDtoDSSP.py 577 2006-10-11 16:51:51Z indraneel $

# converter for PSSD SS definitions from ssd file to dssp file
# ssd and dssp files must exist
# dssp definitions are replaced by pssd definition and written to NEW file

import os
import sys
import string

from Log import LogFH
from PSSDParser import PSSD
from DSSPParser import DSSP

DSSPdir = '../../ddb-dat/dssp/'
SSDdir = '../../ddb-dat/ssd/'
dsspstyleDir = '../../ddb-dat/dsspstyle-ssd/'
logfname = 'x'
overwrite=False

class PSSDtoDSSP:
    def __init__(self, logger):
        self.logger = logger

    def save(self, outfname, overwrite=False):
        logger = self.logger
        #print len(self.convertedLines)
        if not overwrite and os.path.isfile(outfname):
            logger.log("Not overwriting "+outfname)
            return
        fh = file(outfname, 'w')
        fh.writelines(self.convertedLines)
        fh.close()
        
    def convert(self, pssd, dssp):
        convertedLines = dssp.dsspHeader[:]
        chains = dssp.getSortedChains()
        for chainID in chains:
            residues = dssp.getSortedRes(chainID)
            for resID in residues:
                #print chainID, resID, \
                #      pssd[chainID][resID]['SS'], \
                #      dssp[chainID][resID]['SS']
                dsspResidue = dssp[chainID][resID]
                dsspLine = dsspResidue['line']
                if pssd[chainID].has_key(resID):
                    pssdResidue = pssd[chainID][resID]
                    pssdSS = pssdResidue['SS'][0]
                    newline = dsspLine[:16]+pssdSS+dsspLine[17:]
                else:
                    newline = dsspLine
                    logger.log("No "+chainID+str(resID)+" in PSSD")
                convertedLines.append(newline)
                ctr = dsspResidue['#']
                if dssp.dsspSeparators[chainID].has_key(ctr+1):
                    convertedLines.append(dssp.dsspSeparators[chainID][ctr+1])
        self.convertedLines = convertedLines

    def runall(self, SSDdir, DSSPdir, outDir, overwrite=False):
        logger = self.logger
        filelist = os.listdir(SSDdir)
        for fname in filelist:
            ssdFname = SSDdir+fname
            fID = string.join(fname.split('.')[:-1])
            outFname = outDir+fID+'.dssp'
            if not overwrite and os.path.isfile(outFname): continue
            logger.log("processing "+fname)
            dsspFname = DSSPdir+fID+'.dssp'
            if not os.path.isfile(dsspFname):
                logger.log("DSSP missing for "+fID)
                continue
            pssd = PSSD(logger)
            pssd.store(ssdFname)
            dssp = DSSP(logger)
            dssp.store(dsspFname)
            self.convert(pssd, dssp)
            self.save(outFname, overwrite)

    def test(self):
        pssd = PSSD(logger)
        pssd.store(sys.argv[1])
        #pssd.show(sys.stdout)
        dssp = DSSP(logger)
        dssp.store(sys.argv[2])
        #dssp.show(sys.stdout)
        p2d.convert(pssd, dssp)
        p2d.save(sys.argv[3], overwrite=True)

if __name__ == '__main__':
    logger = LogFH(logfname)
    p2d = PSSDtoDSSP(logger)
    if len(sys.argv) == 4:
        p2d.test()
    elif len(sys.argv) > 1:
        print "PSSDtoDSSP.py <ssd file> <dssp file> <output file>"
    else:
        p2d.runall(SSDdir, DSSPdir, dsspstyleDir, overwrite)
    logger.close()

