# $Id: Sheet.py 651 2007-10-10 22:06:22Z indraneel $

import os
import time

from Element import PSSDElement, PSSDElements

class PSSDSheet(PSSDElements):
    def __init__(self, logger):
        self.logger = logger
        PSSDElements.__init__(self, logger)

    def _broken(self, elemi, elemj):
        if elemi in elemj.pairs: return None
        if (elemi.end() in elemj.start().prev[:2]) or \
               (elemi.end() in elemj.start().next[:3]):
            return 'Broken'
        return None

    def _get_broken_pairs(self):
        broken_pairs = []
        self.sort_by_position()
        for i in range(len(self)-1):
            if self._broken(self[i], self[i+1]):
                broken_pairs.append([self[i], self[i+1]])
        return broken_pairs

    def _propagate_brks(self):
        broken_pairs = self._get_broken_pairs()
        if not broken_pairs: return
        for pairs in broken_pairs:
            for pair in pairs:
                pair.log('broken in same sheet')

    def _cmpElemsList(self, elemsA, elemsB):
        # elemsX is [strandi, strandj]
        common = None
        for elem in elemsA:
            if elem in elemsB:
                common = elem
                break
        else: # no common strand
            return 0
        # common strand located
        if len(common.pairs) < 3: # no problem
            return 0
        others = []
        for elem in elemsA+elemsB:
            if elem == common: continue
            others.append(elem)
        # TODO use quad scores
        # (smaller strand preferred (matches link length))
        retval = cmp(len(others[0]), len(others[1]))
        message = 'located common strand '
        message += '(length comparison result: '+str(retval)+'):'
        self.logger.log('info', message)
        common.log(handle='_cmpElemsList')
        for p in common.pairs:
            p.log(handle='_cmpElemsListPairs')
        return retval

    def _setMinor(self):
        linkDict = self.getLinkDict()
        elemDict = {}
        for elem in self:
            elemDict[str(elem)] = 0
            elem.minor = False
        kys = linkDict.keys()
        kys.sort()
        kys.reverse()
        def _cmp(a, b):
            return cmp(a.start().strpos, b.start().strpos)
        def _conseq(a, b):
            if a.start().strpos > b.start().strpos:
                return _conseq(b, a)
            if a.end().get_parent().id != b.start().get_parent().id:
                return False
            if (b.start().strpos - a.end().strpos) > 6: return False
            if (b.start().id[1] - a.end().id[1]) > 6: return False
            return True
        for links in kys:
            elemsList = linkDict[links]
            elemsList.sort(self._cmpElemsList)
            for elems in elemsList:
                minorLink = False
                for elem in elems:
                    elemDict[str(elem)] += 1
                    if elemDict[str(elem)] > 2:
                        buf = []
                        for e in elem.pairs: buf.append(e)
                        buf.sort(_cmp)
                        conseq = False
                        for (i,e) in enumerate(buf[:-1]):
                            if _conseq(e, buf[i+1]):
                                conseq = True
                        if not conseq:
                            minorLink = True
                            self.logger.log('info', 'minorLink True at '+str(elem))
                if minorLink:
                    if elemDict[str(elems[0])] > elemDict[str(elems[1])]:
                        # elems[1] has <3 links
                        if len(elems[1]) < 5:
                            elems[1].minor = True
                            self.logger.log('info', 'minor '+str(elems[1])+' has <3 links and length <5')
                    elif elemDict[str(elems[0])] < elemDict[str(elems[1])]:
                        # elems[0] has <3 links
                        if len(elems[0]) < 5:
                            elems[0].minor = True
                            self.logger.log('info', 'minor '+str(elems[0])+' has <3 links and length <5')
                    else:
                        # both have >2 links each
                        #TODO (change this)
                        if len(elems[0]) < 5:
                            elems[0].minor = True
                            self.logger.log('info', 'minor '+str(elems[0])+' has >2 links and length <5')
                        elems[0].log(handle='_setMinor_Undecided')
                        if len(elems[1]) < 5:
                            elems[1].minor = True
                            self.logger.log('info', 'minor '+str(elems[1])+' has >2 links and length <5')
                        elems[1].log(handle='_setMinor_Undecided')
        self.logger.log('info', '_setMinor() done')
        
    ## Public Methods

    def grow(self, strand):
        strand.tag = 1
        self.append(strand)
        for pair in strand.pairs:
            if pair.tag == None:
                self.grow(pair)

    def interpret(self):
        self.link_back()
        self.link_back_pairs()
        self._propagate_brks()
        self._setMinor()
        return [self]
        
    def setID(self, sheetID):
        self.sheetID = sheetID
        for strand in self:
            strand.sheetID = sheetID

    def _orient(self, strand, Map):
        for pair in strand.pairs:
            if pair.Orient == None:
                pair.Orient = strand.Orient * Map[strand.getOrient(pair)]
                self._orient(pair, Map)

    def mkOrientations(self):
        for strand in self:
            strand.Orient = None
        Map = {'parallel':1,'antiparallel':-1}
        self[0].Orient = 1
        self._orient(self[0], Map)
        
    def getOrient(self, elem1, elem2):
        Map = {1:'parallel', -1:'antiparallel'}
        orient = Map[elem1.Orient * elem2.Orient]
        return orient

class PSSDSheets(list):
    def __init__(self, logger):
        self.logger = logger

    ## Public Methods

    def log(self):
        for sheet in self:
            self.logger.log('element', 'Sheet: '+sheet.sheetID)
            sheet.log()

    def set_sheetID(self):
        sheetID = 1
        for sheet in self:
            sheet.setID(str(sheetID))
            sheetID += 1

    def interpret(self, definition):
        definition.pairup_strands()
        for elem in definition:
            if elem.type == 'E':
                elem.tag = None
                elem.major = False
        sheets = []
        for elem in definition:
            if elem.type != 'E': continue
            if elem.tag == None:
                sheet = PSSDSheet(self.logger)
                sheet.grow(elem)
                sheets.append(sheet)
        for sheet in sheets:
            self += sheet.interpret()
        self.set_sheetID()
        self.logger.log('info', 'Sheets Interpreted')
